/*
  Programmme pour mettre le circuit blanc en mode automatique.
  Franck Perret / Eric Duvieilbourg
*/
#include "RTClib.h"  // bibliothèque horloge rtc
#include "ds3231.h"
#include <Wire.h>  // bibliothèque communication wire
#include <Arduino_MKRENV.h>
#include <SD.h>  // bibliothèque écriture carte SD
#include <SPI.h>  // bibliothèque bus SPI (utilisé pour carte SD)

#include <MKRWAN.h>
#include "arduino_secrets.h"

RTC_DS3231 rtc;  // PCF8523 rtc; ou DS3231 // déclaration RTC

DateTime Dtime;

LoRaModem modem;

// Please enter your sensitive data in the Secret tab or arduino_secrets.h
String appEui = SECRET_APP_EUI;
String appKey = SECRET_APP_KEY;

typedef struct __attribute__ ((__packed__)) { 
  uint32_t timestamp;  // 4 bytes
  int16_t temperature;  // 2 bytes. -400 to +1200
  uint8_t humidity;  // 1 byte   0 to 100
  int16_t pressure;  // 2 bytes
  int16_t illuminance;  // 2 bytes
} SensorDataStruct;  // sensor_data_t

SensorDataStruct sensor;  // creation of the payload structure
int connected;

const int SDCARD_CS = 4;  // Défini la carte SD sur le port 4
const int PIN_LED_BUILTIN = 6;
const int PIN_CMD_TRANSISTOR = 7;

uint8_t ui8_sleep_period = 3;  // the sleep interval in minutes between 2 consecutive alarms

void set_next_alarm(void)
{
  struct ts S_t;

  DS3231_get(&S_t);

  // calculate the minute when the next alarm will be triggered
  unsigned char uc_wakeup_min = (S_t.min / ui8_sleep_period + 1) * ui8_sleep_period;
  if (uc_wakeup_min > 59) {
    uc_wakeup_min -= 60;
  }

  // flags define what calendar component to be checked against the current time in order
  // to trigger the alarm
  // A2M2 (minutes) (0 to enable, 1 to disable)
  // A2M3 (hour)    (0 to enable, 1 to disable)
  // A2M4 (day)     (0 to enable, 1 to disable)
  // DY/DT          (dayofweek == 1/dayofmonth == 0)
  uint8_t ui8_flags[4] = { 0, 1, 1, 1 };

  // set Alarm2. only the mn is set since we ignore the hour and day component
  DS3231_set_a2(uc_wakeup_min, 0, 0, ui8_flags);

  // activate Alarm2
  DS3231_set_creg(DS3231_CONTROL_INTCN | DS3231_CONTROL_A2IE);
}


void setup()
{
  rtc.begin();                                                                                                           //lance le RTC  
  pinMode(PIN_LED_BUILTIN, OUTPUT);
  digitalWrite(PIN_LED_BUILTIN, LOW);                                                                                                //Sortie pin led                       
  pinMode(PIN_CMD_TRANSISTOR, OUTPUT);                                                                                        //Sortie pin 7 HIGH vers transistor
  digitalWrite(PIN_CMD_TRANSISTOR, HIGH);

  delay(200);

  DS3231_init(DS3231_CONTROL_INTCN);
    DS3231_clear_a1f();
    DS3231_clear_a2f();

  // following line sets the RTC to the date & time this sketch was compiled
  //   rtc.adjust(DateTime(F(__DATE__), F(__TIME__)));

  //   rtc.adjust(DateTime(2020, 1, 13, 16, 20, 0));  // Défini une date et une heure pour le RTC (à ne faire qu'une fois)
  // Par exemple pour le 9 avril 2018 à 16:36:20
  // rtc.adjust(DateTime(2018, 4 ,9 , 16 ,36 ,20));

   if (!ENV.begin()) {
    // Bloque le programme si probleme
    while (1){
      digitalWrite(PIN_LED_BUILTIN, HIGH); delay(500);
      digitalWrite(PIN_LED_BUILTIN, LOW); delay(500);
    }
  }
  digitalWrite(PIN_LED_BUILTIN, HIGH);


  if (!SD.begin(SDCARD_CS)) {
    while (1) {
      digitalWrite(PIN_LED_BUILTIN, HIGH); delay(200);
      digitalWrite(PIN_LED_BUILTIN, LOW); delay(300);
    }
  }
  digitalWrite(PIN_LED_BUILTIN, HIGH);
 
  set_next_alarm();

  if (SD.exists("datalog.txt")) {  // gestion de l'append
    File fdata = SD.open("datalog.txt", FILE_WRITE);
    fdata.close();
  }
  else {
    // Ouvre/crée un fichier nommé "datalog" dans la carte SD
    File fdata = SD.open("datalog.txt", FILE_WRITE);  
    // Ecriture dans le fichier de l'entête des données
    fdata.println("Date, Heure, Temperature, Humidity, Pressure, Illuminance");
    // Ferme le fichier de la carte SD
    fdata.close();  
  }

  // change this to your regional band (eg. US915, AS923, ...)
  if (modem.begin(EU868)) {
    connected = modem.joinOTAA(appEui, appKey);
    if (connected) {
      modem.minPollInterval(ui8_sleep_period * 60);
    }
  }    

}

void loop() {

  delay(3000);  // attente 3s
  
  Dtime = rtc.now();  // Défini la date actuelle comme celle du RTC
  // Dtime = DateTime(y, m, d, hh, mm, ss) on veut day/month/year, hh.mm,

  // read all the sensor values
  sensor = {
      .timestamp = (uint32_t) Dtime.unixtime(),
      .temperature = (int16_t) (ENV.readTemperature() * 10),  // x 10
      .humidity = (uint8_t) (ENV.readHumidity()),  // hum in %
      .pressure = (int16_t) (ENV.readPressure(MILLIBAR)),  // en hectoP
      .illuminance = (int16_t) (ENV.readIlluminance() * 10)  // x 10
  };

  if (connected) {
    modem.beginPacket();
    modem.write((uint8_t *) &sensor, sizeof(sensor));  // cf. LoRaModem.h
    int err = modem.endPacket(true);
    if (err > 0) {
    //// Serial.println("Message sent correctly!");
      for(int i=0; i<3; i++) {
        digitalWrite(PIN_LED_BUILTIN, HIGH); delay(800);
        digitalWrite(PIN_LED_BUILTIN, LOW); delay(200);
      }      
    } else {
      for(int i=0; i<3; i++) {
        digitalWrite(PIN_LED_BUILTIN, HIGH); delay(200);
        digitalWrite(PIN_LED_BUILTIN, LOW); delay(800);
      }      
    }  
  }

  char mlog_dtime[20], mlog[128];

  sprintf(mlog_dtime, "%d/%d/%d, %d.%d, ", \
                      Dtime.day(), Dtime.month(), Dtime.year(), \
                      Dtime.hour(), Dtime.minute());
  sprintf(mlog, "%.1f, %d, %d, %.2f", ((float)(sensor.temperature) / 10), \
                                      (int)(sensor.humidity), \
                                      (int)(sensor.pressure), \
                                      ((float)(sensor.illuminance) / 10));

  // Ouvre/crée un fichier nommé "datalog" dans la carte SD
  File fdata = SD.open("datalog.txt", FILE_WRITE);  
  fdata.print(mlog_dtime);
  fdata.println(mlog);
  fdata.close();
                       
  while (1) {
    digitalWrite(PIN_CMD_TRANSISTOR, LOW);                                                                                   //Sortie pin 7 LOW  POUR MISE EN SOMMEIL DE LA CARTE MKR
    delay(5000);  // attente bloquante de 5 sec
  }
}
